#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
//#include <SGL.h> --> Update 2017-08-02 : Ditched SGL by just include what's needed in the Def.h file. Some features might be broken, need more testing. If you encounter problems, comment #include "Def.h" and use SGL.h instead (but make sure you point to the file)
#include "Def.h"
/*********************************************************
* Bounding box generator by XL2. Calculates slopes as well. To be used with a 3D game on the Sega Saturn. Just use a function similar to the jo_square_intersect function, but add
* an extra variable for the Z-axis. If there is a slope, refer to a height collision mask to see if there is a collision of not.
* Make sure that you include SGL (SGL.h) in your project! (UPDATE 2017-08-02 : Removed the need to use SGL. Might cause issues and need more testing)
* Use the display command to load the map, then compile and run everything (output is currently set in a .txt file)
* You will have to manually set the path yourself.
* See below (currently at line 103), you must include your JO_ENGINE 3d mesh .h file
* Make sure that your polygons all have normals as well, since it will be used to determine the direction of the polygon and add depth/slopes. You might have to edit your .h file to remove textures and other things (just comment them out)
* TO DO : Auto-collision mask generation
* TO DO : Code clean up (I commented out several lines for my tests with binary writing and other functions)
* TO DO : Merge more bounding boxes to reduce the CPU load.
* TO DO : Add proper BSP
* TO DO : Write to binary to load the map and its collision data on the Saturn on map change
* TO DO : More testing to find out bugs
***********************************/
using namespace std;

typedef struct
{
    PDATA       data;
}               jo_3d_mesh;
jo_3d_mesh mesh_array[200] = {0};
#include "OBJ2SGL.h"
/***************
Since the JO file math.h has the same name as the C++ function, it creates issues with the compiler. So I just import the atan2 function as a
quick and dirty workaround
**************/
float       atan2f_rad(const float y, const float x)
{
    float   atan;
    float   z;
    float JO_PI_2 = 1.5707963;
    float JO_PI = 3.1415927;

    if ((x<0.00001f) && x>-0.00001f)

		if (y > 0.0f)
            return (JO_PI_2);
		if ((y<0.00001f) && (y>-0.00001f))
            return 0.0f;
		return (-JO_PI_2);

	z = y / x;
	if (z < 1.0f && z > -1.0f)
	{
		atan = z / (1.0f + 0.28f * z * z);
		if (x < 0.0f)
		{
			if (y < 0.0f)
                return (atan - JO_PI);
			return (atan + JO_PI);
		}
	}
	else
	{
		atan = JO_PI_2 - z / (z * z + 0.28f);
		if (y < 0.0f)
            return (atan - JO_PI);
	}
	return (atan);


}

unsigned short total_mesh = 0;
/******************************************************
 If you are writing only the collision data, put this to TRUE. All the bounding boxes will be included in the level. Slopes will need to be handcoded with masks
 ******************************************************/
bool is_BB_mesh = false;




/*Just to prevent errors being returned without having to modify your 3D mesh .h file.
Just use the display function to initialize all the meshes. This will be used for the bounding boxes*/
void jo_3d_mesh_draw(jo_3d_mesh * mesh)
{

        mesh_array[total_mesh].data = mesh->data;
        total_mesh ++;

}

#define __jo_force_inline inline

/*******************
//Put the JO ENGINE 3d mesh files you want to read here. It can have several meshes in one file.
The display_mesh command within the JO ENGINE file will be used to create an array of all the meshes in the file.
From that you will be able to write to the binary file and create bounding boxes that will be used for your binary space partitioning.
This allows you to translate/scale/rotate only the vertices within your camera or avatar's vicinity, which helps a lot
for the Saturn CPU. Use the "UseNearClip" command in your Mesh file to further help the Saturn VDP1 by not drawing offscreen quads.
*******************/
/******************************************************/

#include "SGL/FPSlevel.h"
/******************************************************/
#include "MAP2WRAM.h"
#include "Map2Bin.h"

typedef struct
{
    int x;
    int y;
    int z;
} vector3;

/* Masks are used for non-axis aligned objects. It takes more RAM than calculating slopes on the fly, but it saves CPU usage. For more complex map layout, masks should be avoided */
int total_Masks = 0;
int Mask[100][200]={0};

typedef struct
{
    vector3 MaxValue;
    vector3 MinValue;
    vector3 Rotation;
    int     MaskID;
} _BB_Hitbox;


_BB_Hitbox BB_Hitbox[300] = {0};
//_BB_Hitbox BSP_Hitbox;


/******************************
This function prevents having false data for the BSP Hitbox data, like if all your points are > 0, then your minimum value would still be 0.
By putting everything to their opposite value, we make sure that the min/max values are correct
******************************/
/*void initialize_BSP_hitbox(void)
{
    BSP_Hitbox.MinValue.x = {2147483647};
    BSP_Hitbox.MinValue.y = {2147483647};
    BSP_Hitbox.MinValue.z = {2147483647};
    BSP_Hitbox.MaxValue.x = {-2147483647};
    BSP_Hitbox.MaxValue.y = {-2147483647};
    BSP_Hitbox.MaxValue.z = {-2147483647};
}*/

/******************************
This function prevents having false data, like if all your points are > 0, then your minimum value would still be 0.
By putting everything to their opposite value, we make sure that the min/max values are correct
******************************/
/*void initialize_BB_hitbox(void)
{
    for (int i = 0; i < total_mesh; i++)
    {
        BB_Hitbox[i].MinValue.x = {2147483647};
        BB_Hitbox[i].MinValue.y = {2147483647};
        BB_Hitbox[i].MinValue.z = {2147483647};
        BB_Hitbox[i].MaxValue.x = {-2147483647};
        BB_Hitbox[i].MaxValue.y = {-2147483647};
        BB_Hitbox[i].MaxValue.z = {-2147483647};
    }
}*/

/*
void initialize_global_bb_hitbox(void)
{
    for (int i = 0; i < 20; i++)
    {
        for (int ii = 0; ii < 3000; ii++)
        {
            GLOBAL_Hitbox[i].BB_Hitbox[ii].MinValue.x = {2147483647};
            GLOBAL_Hitbox[i].BB_Hitbox[ii].MinValue.y = {2147483647};
            GLOBAL_Hitbox[i].BB_Hitbox[ii].MinValue.z = {2147483647};
            GLOBAL_Hitbox[i].BB_Hitbox[ii].MaxValue.x = {-2147483647};
            GLOBAL_Hitbox[i].BB_Hitbox[ii].MaxValue.y = {-2147483647};
            GLOBAL_Hitbox[i].BB_Hitbox[ii].MaxValue.z = {-2147483647};
        }
        GLOBAL_Hitbox[i].LengthBBValues = 0;
        GLOBAL_Hitbox[i].BB_Boundaries.MinValue.x = {2147483647};
        GLOBAL_Hitbox[i].BB_Boundaries.MinValue.y = {2147483647};
        GLOBAL_Hitbox[i].BB_Boundaries.MinValue.z = {2147483647};
        GLOBAL_Hitbox[i].BB_Boundaries.MaxValue.x = {-2147483647};
        GLOBAL_Hitbox[i].BB_Boundaries.MaxValue.y = {-2147483647};
        GLOBAL_Hitbox[i].BB_Boundaries.MaxValue.z = {-2147483647};

    }


}*/

/******************************************************************
The BSP Hitbox is used for when we draw the data. That means that whatever assets you have in your .h file, they should be close to each other because
you will use the BSP hitbox to display them or not depending on their distance from your avatar/camera in order to reduce CPU load since it won't need
to do matrix translations/rotations/scaling
******************************************************************/
/*void generate_BSP_hitbox(void)
{

    initialize_BSP_hitbox();
    if (is_BB_mesh == true) return;  //Since we include all the Bounding boxes from the level, which is low (usually less than 200), we don't really need binary space partitioning (for sake of simplicity, it's disabled)
    for (int meshID = 0; meshID < total_mesh; meshID++)
    {
        for (int i = 0; i < mesh_array[meshID].data.nbPoint; i++)
        {
          if (mesh_array[meshID].data.pntbl[i][0] < BSP_Hitbox.MinValue.x) BSP_Hitbox.MinValue.x = mesh_array[meshID].data.pntbl[i][0];
          if (mesh_array[meshID].data.pntbl[i][0] > BSP_Hitbox.MaxValue.x) BSP_Hitbox.MaxValue.x = mesh_array[meshID].data.pntbl[i][0];
          if (mesh_array[meshID].data.pntbl[i][1] < BSP_Hitbox.MinValue.y) BSP_Hitbox.MinValue.y = mesh_array[meshID].data.pntbl[i][1];
          if (mesh_array[meshID].data.pntbl[i][1] > BSP_Hitbox.MaxValue.y) BSP_Hitbox.MaxValue.y = mesh_array[meshID].data.pntbl[i][1];
          if (mesh_array[meshID].data.pntbl[i][2] < BSP_Hitbox.MinValue.z) BSP_Hitbox.MinValue.z = mesh_array[meshID].data.pntbl[i][2];
          if (mesh_array[meshID].data.pntbl[i][2] > BSP_Hitbox.MaxValue.z) BSP_Hitbox.MaxValue.z = mesh_array[meshID].data.pntbl[i][2];
        }
    }
}*/


/******************************************
The Bounding Box hitbox is for your avatar's collision detection. You should use axis-aligned bounding boxes with your 3D editor software (like Blender),
align them on the "real" polygons, convert everything to JO ENGINE compatible .h file. Keep it low (less than 200 should be enough in most scenarios).
These boxes in the .h file WON'T be used by Saturn : you only need them to compile a file with all the bounding boxes, which will be sent to the Saturn by putting it in your c code
(or importing it via .bin files). The slopes aren't calculated yet.
******************************************/
/*void generate_BB_hitbox(string filename)
{
    string name;
    name= filename+".txt";
    initialize_BB_hitbox();
    for (int meshID = 0; meshID < total_mesh; meshID++)
    {
        for (int i = 0; i < mesh_array[meshID].data.nbPoint; i++)
        {
          if (mesh_array[meshID].data.pntbl[i][0] < BB_Hitbox[meshID].MinValue.x) BB_Hitbox[meshID].MinValue.x = mesh_array[meshID].data.pntbl[i][0];
          if (mesh_array[meshID].data.pntbl[i][0] > BB_Hitbox[meshID].MaxValue.x) BB_Hitbox[meshID].MaxValue.x = mesh_array[meshID].data.pntbl[i][0];
          if (mesh_array[meshID].data.pntbl[i][1] < BB_Hitbox[meshID].MinValue.y) BB_Hitbox[meshID].MinValue.y = mesh_array[meshID].data.pntbl[i][1];
          if (mesh_array[meshID].data.pntbl[i][1] > BB_Hitbox[meshID].MaxValue.y) BB_Hitbox[meshID].MaxValue.y = mesh_array[meshID].data.pntbl[i][1];
          if (mesh_array[meshID].data.pntbl[i][2] < BB_Hitbox[meshID].MinValue.z) BB_Hitbox[meshID].MinValue.z = mesh_array[meshID].data.pntbl[i][2];
          if (mesh_array[meshID].data.pntbl[i][2] > BB_Hitbox[meshID].MaxValue.z) BB_Hitbox[meshID].MaxValue.z = mesh_array[meshID].data.pntbl[i][2];
        }
    }
    ofstream ofile(name.c_str(), ios::out | ios::ate);


    ofile << "#ifndef __ZONE1BB_H__ \n#define __ZONE1_BB_H\n";
    ofile << "#include ''../../level.h''\n\n";
    ofile << "#define zone_total_blocks (" << total_mesh << ")\n\n";

    ofile << "const grids const zone_bb[] =\n{\n";
    for (int meshID = 0; meshID < total_mesh; meshID ++)
    {
        //Here we round up to Int value (since it doesn't keep decimals, we add/remove 0.5 to round up to the superior value)
        if (BB_Hitbox[meshID].MinValue.x < 0) BB_Hitbox[meshID].MinValue.x = (BB_Hitbox[meshID].MinValue.x -32768)/65536;
            else BB_Hitbox[meshID].MinValue.x = (BB_Hitbox[meshID].MinValue.x + 32768)/65536;
        if (BB_Hitbox[meshID].MinValue.y < 0) BB_Hitbox[meshID].MinValue.y = (BB_Hitbox[meshID].MinValue.y-32768)/65536;
            else BB_Hitbox[meshID].MinValue.y = (BB_Hitbox[meshID].MinValue.y + 32768)/65536;
        if (BB_Hitbox[meshID].MinValue.z < 0) BB_Hitbox[meshID].MinValue.z = (BB_Hitbox[meshID].MinValue.z-32768)/65536;
            else BB_Hitbox[meshID].MinValue.z = (BB_Hitbox[meshID].MinValue.z + 32768)/65536;
        if (BB_Hitbox[meshID].MaxValue.x < 0) BB_Hitbox[meshID].MaxValue.x = (BB_Hitbox[meshID].MaxValue.x - 32768)/65536;
            else BB_Hitbox[meshID].MaxValue.x = (BB_Hitbox[meshID].MaxValue.x + 32768)/65536;
        if (BB_Hitbox[meshID].MaxValue.y < 0) BB_Hitbox[meshID].MaxValue.y = (BB_Hitbox[meshID].MaxValue.y - 32768)/65536;
            else BB_Hitbox[meshID].MaxValue.y = (BB_Hitbox[meshID].MaxValue.y + 32768)/65536;
        if (BB_Hitbox[meshID].MaxValue.z < 0) BB_Hitbox[meshID].MaxValue.z = (BB_Hitbox[meshID].MaxValue.z - 32768)/65536;
            else BB_Hitbox[meshID].MaxValue.z = (BB_Hitbox[meshID].MaxValue.z + 32768)/65536;

        ofile << "   {";
        ofile << BB_Hitbox[meshID].MinValue.x << ", " << (BB_Hitbox[meshID].MaxValue.x - BB_Hitbox[meshID].MinValue.x) << ", ";
        ofile << BB_Hitbox[meshID].MinValue.y << ", " << (BB_Hitbox[meshID].MaxValue.y - BB_Hitbox[meshID].MinValue.y) << ", ";
        ofile << BB_Hitbox[meshID].MinValue.z << ", " << (BB_Hitbox[meshID].MaxValue.z - BB_Hitbox[meshID].MinValue.z) << ", ";
        ofile << "-1, 0, 0},\n";
    }
    ofile << "};\n#endif";
    ofile.close();

    name=filename+".bin";
    ofstream obinfile(name.c_str(), ios::out | ios::binary);

    if (obinfile.is_open())
    {
        for (int meshID = 0; meshID < total_mesh; meshID ++)
        {
            char minx[4] = {(BB_Hitbox[meshID].MinValue.x)};
            char miny[4] = {BB_Hitbox[meshID].MinValue.y};
            char minz[4] = {BB_Hitbox[meshID].MinValue.z};
            char lenx[4] = {BB_Hitbox[meshID].MaxValue.x - BB_Hitbox[meshID].MinValue.x};
            char leny[4] = {BB_Hitbox[meshID].MaxValue.y - BB_Hitbox[meshID].MinValue.y};
            char lenz[4] = {BB_Hitbox[meshID].MaxValue.z - BB_Hitbox[meshID].MinValue.z};

            obinfile.write (minx, 2);
            obinfile.write (lenx, 2);
            obinfile.write (miny, 2);
            obinfile.write(leny, 2);
            obinfile.write(minz, 2);
            obinfile.write(lenz, 2);
        }
    }

    obinfile.close();


}*/



    vector3 list_vertices[3000] = {0};
    _BB_Hitbox list_per_poly_bb[3000];
    int list_vertices_total_points = 0;
    int list_polygons_total_poly = 0;

/*************
Makes a list of all the vertices in your file. Since the Jo Engine Map editor outputs ID that increase between different meshes, it
allows us to keep a global list.
**************/
void fill_list_vertices(void)
{


    for (int meshID = 0; meshID < total_mesh; meshID++)
    {

        for (int i = 0; i < mesh_array[meshID].data.nbPoint; i++)
        {
            list_vertices[list_vertices_total_points].x = mesh_array[meshID].data.pntbl[i][0];
            list_vertices[list_vertices_total_points].y = mesh_array[meshID].data.pntbl[i][1];
            list_vertices[list_vertices_total_points].z = mesh_array[meshID].data.pntbl[i][2];
            list_vertices_total_points ++;
        }
    }

   /* cout << "\n*************CONFIRMATION OF LIST OF VERTICES*******************\n";
    for (int i=0; i < list_vertices_total_points; i++)
    {
        cout << "X : " << list_vertices[i].x << " Y : " << list_vertices[i].y << " Z : " << list_vertices[i].z;
    }*/

}

/*
void deletevalues(int i)
{

                      for (int j=i; j< list_polygons_total_poly-1; j++)
               {
                    poly_bb[j].MaxValue.x = poly_bb[j+1].MaxValue.x; poly_bb[j].MaxValue.y = poly_bb[j+1].MaxValue.y; poly_bb[j].MaxValue.z = poly_bb[j+1].MaxValue.z;
                    poly_bb[j].MinValue.x = poly_bb[j+1].MinValue.x; poly_bb[j].MinValue.y = poly_bb[j+1].MinValue.y; poly_bb[j].MinValue.z = poly_bb[j+1].MinValue.z;

               }



}*/

void CreateMask(_BB_Hitbox buffer)
{

}


_BB_Hitbox poly_bb[3000]={0};
/***************************
To avoid having too many bounding boxes, we attempt to regroup them here. If it returns > 0, that means there is a duplicate and we won't write it in the file, but
we will update it to merge the boxes. Right now, it's not efficient enough as it might reduce a file with 2 300 polygons to 900 boxes, but by manually writing bounding boxes you
might be able to reduce that number to 100.
***************************/
int refine_poly_2_bb_list(_BB_Hitbox *passedBuffer)
{
    if ((passedBuffer->Rotation.x !=0 && passedBuffer->Rotation.x != 90 && passedBuffer->Rotation.x != -90) || (passedBuffer->Rotation.z != 0 && passedBuffer->Rotation.z != 90 && passedBuffer->Rotation.z != -90)) return 0;
    int merged = 0; /********** RETURNS AMOUNT OF MERGED BOXES **/
    for (int i=0; i < list_polygons_total_poly; i++)
    {

        if ((poly_bb[i].MaxValue.x == passedBuffer->MinValue.x) && (poly_bb[i].MaxValue.y == passedBuffer->MaxValue.y && poly_bb[i].MinValue.y == passedBuffer->MinValue.y) && (poly_bb[i].MaxValue.z == passedBuffer->MaxValue.z && poly_bb[i].MinValue.z == passedBuffer->MinValue.z))
        {
            poly_bb[i].MaxValue.x = passedBuffer->MaxValue.x;
            merged ++;
        }
        else if ((poly_bb[i].MinValue.x == passedBuffer->MaxValue.x) && (poly_bb[i].MaxValue.y == passedBuffer->MaxValue.y && poly_bb[i].MinValue.y == passedBuffer->MinValue.y) && (poly_bb[i].MaxValue.z == passedBuffer->MaxValue.z && poly_bb[i].MinValue.z == passedBuffer->MinValue.z))
        {
            poly_bb[i].MinValue.x = passedBuffer->MinValue.x;
            merged ++;
        }
        else if (poly_bb[i].MaxValue.y == passedBuffer->MinValue.y && (poly_bb[i].MinValue.x == passedBuffer->MinValue.x && poly_bb[i].MaxValue.x == passedBuffer->MaxValue.x) && (poly_bb[i].MaxValue.z == passedBuffer->MaxValue.z && poly_bb[i].MinValue.z == passedBuffer->MinValue.z))
        {
            poly_bb[i].MaxValue.y = passedBuffer->MaxValue.y;
            merged ++;
        }
        else if (poly_bb[i].MinValue.y == passedBuffer->MaxValue.y && (poly_bb[i].MinValue.x == passedBuffer->MinValue.x && poly_bb[i].MaxValue.x == passedBuffer->MaxValue.x) && (poly_bb[i].MaxValue.z == passedBuffer->MaxValue.z && poly_bb[i].MinValue.z == passedBuffer->MinValue.z))
        {
            poly_bb[i].MinValue.y = passedBuffer->MinValue.y;
            merged ++;
        }
        else if ((poly_bb[i].MaxValue.z == passedBuffer->MinValue.z) && (poly_bb[i].MinValue.x == passedBuffer->MinValue.x && poly_bb[i].MaxValue.x == passedBuffer->MaxValue.x) && (poly_bb[i].MaxValue.y == passedBuffer->MaxValue.y && poly_bb[i].MinValue.y == passedBuffer->MinValue.y))
        {
            poly_bb[i].MaxValue.z = passedBuffer->MaxValue.z;
            merged ++;
        }
        else if ((poly_bb[i].MinValue.z == passedBuffer->MaxValue.z) && (poly_bb[i].MinValue.x == passedBuffer->MinValue.x && poly_bb[i].MaxValue.x == passedBuffer->MaxValue.x) && (poly_bb[i].MaxValue.y == passedBuffer->MaxValue.y && poly_bb[i].MinValue.y == passedBuffer->MinValue.y))
        {
            poly_bb[i].MinValue.z = passedBuffer->MinValue.z;
            merged ++;
        }


    }


       return merged;
}



/****************************************
Generate collision BB based on polygons. Generates slopes as well depending on the angle, but
doesn't generate the heigth masks yet (right now it only outputs -1 (no mask), 0 and 1 (different masks, depending on the direction).
TODO : Proper BSP (I had to remove it because some BB got cut in two, which made you "fall" ingame.
TODO : Generate collision height masks.
TODO : Better merging of the BB, while also avoiding losing the slopes' data.
*****************************************/
void poly_2_BB(string filename)
{
   cout << "\nPOLYGON TO BOUNDING BOX - BETA\n";
   ofstream file("TESTPOLY2BB.txt", ios::out);

    fill_list_vertices();



    if (file.is_open())
    {

        for (int mesh = 0; mesh < total_mesh; mesh++)
        {
            //list_polygons_total_poly = 0;
            //file << "\nMesh no." << mesh << "\n"; -> If you use the Meshes for your BSP. I removed it because of a bug that cut BB in two and lost data.
            for (int i=0; i<mesh_array[mesh].data.nbPolygon; i++)
            {
               // int minX = 2147483647;                int minY = 2147483647;                int minZ = 2147483647;                int maxX = -2147483647;                int maxY = -2147483647;                int maxZ = -2147483647;
                _BB_Hitbox buffer;
                buffer.MinValue.x = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[0]].x;
                buffer.MinValue.y = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[0]].y;
                buffer.MinValue.z = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[0]].z;
                buffer.MaxValue.x = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[0]].x;
                buffer.MaxValue.y = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[0]].y;
                buffer.MaxValue.z = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[0]].z;

                for (int ii=0; ii<4; ii++)
                {
                   // cout << "\n*************************\n" << mesh_array[mesh].data.pltbl[i].Vertices[ii];
                    if (list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].x < buffer.MinValue.x)
                        buffer.MinValue.x = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].x;
                    if (list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].x > buffer.MaxValue.x)
                        buffer.MaxValue.x = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].x;
                    if (list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].y < buffer.MinValue.y)
                        buffer.MinValue.y = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].y;
                    if (list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].y > buffer.MaxValue.y)
                        buffer.MaxValue.y = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].y;
                    if (list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].z < buffer.MinValue.z)
                        buffer.MinValue.z = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].z;
                    if (list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].z > buffer.MaxValue.z)
                        buffer.MaxValue.z = list_vertices[mesh_array[mesh].data.pltbl[i].Vertices[ii]].z;

                }

                if (buffer.MinValue.x > 0) {buffer.MinValue.x = (buffer.MinValue.x + 32768)>>16;}
                else  {buffer.MinValue.x = (buffer.MinValue.x - 32768)>>16;}
                if (buffer.MinValue.y > 0) {buffer.MinValue.y = (buffer.MinValue.y + 32768)>>16;}
                else  {buffer.MinValue.y = (buffer.MinValue.y - 32768)>>16;}
                if (buffer.MinValue.z > 0) {buffer.MinValue.z = (buffer.MinValue.z + 32768)>>16;}
                else  {buffer.MinValue.z = (buffer.MinValue.z - 32768)>>16;}
                if (buffer.MaxValue.x > 0) {buffer.MaxValue.x = (buffer.MaxValue.x + 32768)>>16;}
                else  {buffer.MaxValue.x = (buffer.MaxValue.x - 32768)>>16;}
                if (buffer.MaxValue.y > 0) {buffer.MaxValue.y = (buffer.MaxValue.y + 32768)>>16;}
                else  {buffer.MaxValue.y = (buffer.MaxValue.y - 32768)>>16;}
                if (buffer.MaxValue.z > 0) {buffer.MaxValue.z = (buffer.MaxValue.z + 32768)>>16;}
                else  {buffer.MaxValue.z = (buffer.MaxValue.z - 32768)>>16;}

                if (buffer.MaxValue.x - buffer.MinValue.x == 0)
                    {
                       if (mesh_array[mesh].data.pltbl[i].norm[0] > 0) {buffer.MinValue.x -= 50;}
                       else if (mesh_array[mesh].data.pltbl[i].norm[0] < 0) {buffer.MaxValue.x += 50;}
                    }
                if (buffer.MaxValue.y - buffer.MinValue.y == 0)
                    {
                        if (mesh_array[mesh].data.pltbl[i].norm[1] > 0) buffer.MinValue.y -= 50;
                        else if (mesh_array[mesh].data.pltbl[i].norm[1] < 0) buffer.MaxValue.y += 50;
                    }
                if (buffer.MaxValue.z - buffer.MinValue.z == 0)
                    {
                        if (mesh_array[mesh].data.pltbl[i].norm[2] > 0) buffer.MinValue.z -= 50;
                        else if (mesh_array[mesh].data.pltbl[i].norm[2] < 0) buffer.MaxValue.z += 50;
                    }

                if (mesh_array[mesh].data.pltbl[i].norm[0]!=0 && mesh_array[mesh].data.pltbl[i].norm[1]!=0)  //Slope along the Z axis, we need to calculate it's value.
                {
                    buffer.Rotation.x = 0;
                    //int hypotenuse = jo_sqrt(((buffer.MaxValue.x-buffer.MinValue.x)^2) + ((buffer.MaxValue.y - buffer.MinValue.y)^2));
                    float slopeAngle;
                    if ((buffer.MaxValue.x-buffer.MinValue.x)==(buffer.MaxValue.y-buffer.MinValue.y))
                        slopeAngle=45;
                    else {slopeAngle = atan2f_rad(buffer.MaxValue.y-buffer.MinValue.y,buffer.MaxValue.x-buffer.MinValue.x);
                    slopeAngle = slopeAngle * (180.f/3.1415927);}
                    int slopeAngle_Int;
                    if (slopeAngle < 0.0f) slopeAngle_Int = (slopeAngle)-0.5;
                    else slopeAngle_Int = (slopeAngle)+0.5;
                    cout << "SLOPE ANGLE = " << slopeAngle_Int << "\n";
                    cout << "SLOPE ANGLE_= " << slopeAngle << "\n";

                    if (slopeAngle_Int < 10 && slopeAngle_Int > -10) //Avoid wasting memory with negligible slopes due to mistakes made with 3D tools
                    {
                        buffer.Rotation.z = 0;
                        buffer.MaskID = -1;
                    }
                    else if (mesh_array[mesh].data.pltbl[i].norm[0]< 0)
                    {
                        buffer.Rotation.z = -slopeAngle_Int; // (need more tests, put negative or positive value here)
                        buffer.MaskID = 1;
                    }
                    else
                    {
                        buffer.Rotation.z = slopeAngle_Int;
                        buffer.MaskID = 0;

                    }//buffer.MaskID = total_Masks;
                }
                else if (mesh_array[mesh].data.pltbl[i].norm[2]!=0 && mesh_array[mesh].data.pltbl[i].norm[1]!=0) //Slope along the X axis
                {
                    buffer.Rotation.z = 0;
                    float slopeAngle;
                    if ((buffer.MaxValue.z-buffer.MinValue.z)==(buffer.MaxValue.y-buffer.MinValue.y))
                        slopeAngle=45;
                    else
                    {   slopeAngle = atan2f_rad(buffer.MaxValue.y-buffer.MinValue.y,buffer.MaxValue.z-buffer.MinValue.z);
                        slopeAngle = slopeAngle * (180.f/3.1415927); }

                    int slopeAngle_Int;
                    if (slopeAngle < 0.0f) slopeAngle_Int = (slopeAngle)-0.5;
                    else slopeAngle_Int = (slopeAngle)+0.5;
                    cout << "\nSLOPE ANGLE = " << slopeAngle_Int << "\n";

                    if (slopeAngle_Int < 10 && slopeAngle_Int > -10) //Disregard low slopes as they might just be mistakes made with the 3D editor
                    {
                        buffer.Rotation.x = 0;
                        buffer.MaskID = -1;
                    }
                    else if (mesh_array[mesh].data.pltbl[i].norm[2]< 0)
                    {
                        buffer.Rotation.x = slopeAngle_Int;
                        buffer.MaskID = 1;
                    }
                    else
                    {
                        buffer.Rotation.x = -slopeAngle_Int;
                        buffer.MaskID = 0;
                    }
                    //buffer.Rotation.x = slopeAngle_Int;
                    //buffer.MaskID = total_Masks;
                }
                else
                {
                    buffer.Rotation.x = 0;
                    buffer.Rotation.z=0;
                    buffer.MaskID=-1;

                }


                   if (refine_poly_2_bb_list(&buffer) > 0)
                   {
                        //cout << "Refining results. Merging bounding boxes...\n";
                   }
                   else
                   {
                    poly_bb[list_polygons_total_poly].MaxValue.x = buffer.MaxValue.x; poly_bb[list_polygons_total_poly].MaxValue.y = buffer.MaxValue.y; poly_bb[list_polygons_total_poly].MaxValue.z = buffer.MaxValue.z;
                    poly_bb[list_polygons_total_poly].MinValue.x = buffer.MinValue.x; poly_bb[list_polygons_total_poly].MinValue.y = buffer.MinValue.y; poly_bb[list_polygons_total_poly].MinValue.z = buffer.MinValue.z;
                    poly_bb[list_polygons_total_poly].Rotation.x = buffer.Rotation.x; poly_bb[list_polygons_total_poly].Rotation.z = buffer.Rotation.z;
                    poly_bb[list_polygons_total_poly].MaskID=buffer.MaskID;
                    list_polygons_total_poly ++;
                   }

                //cout << "\nVert : " << mesh_array[mesh].data.pntbl[mesh_array[mesh].data.pltbl[i].Vertices[1]][0];
            }





        }

            for (int i = 0; i < list_polygons_total_poly; i++)
            {
                for (int ii = 0; ii < list_polygons_total_poly; ii++)
                {
                    if (poly_bb[i].MaxValue.x == poly_bb[ii].MaxValue.x && poly_bb[i].MinValue.x == poly_bb[ii].MinValue.x && poly_bb[i].MaxValue.y == poly_bb[ii].MaxValue.y && poly_bb[i].MinValue.y == poly_bb[ii].MinValue.y && poly_bb[i].MaxValue.z == poly_bb[ii].MaxValue.z && poly_bb[i].MinValue.z == poly_bb[ii].MinValue.z && (i != ii) && poly_bb[i].Rotation.x ==0 && poly_bb[i].Rotation.z == 0)
                    {
                        for (int j=ii; j<list_polygons_total_poly-1; j++)
                            {
                                poly_bb[j].MaxValue.x = poly_bb[j+1].MaxValue.x; poly_bb[j].MaxValue.y = poly_bb[j+1].MaxValue.y; poly_bb[j].MaxValue.z = poly_bb[j+1].MaxValue.z;
                                poly_bb[j].MinValue.x = poly_bb[j+1].MinValue.x; poly_bb[j].MinValue.y = poly_bb[j+1].MinValue.y; poly_bb[j].MinValue.z = poly_bb[j+1].MinValue.z;
                            }
                                list_polygons_total_poly -= 1;
                    }
                }
            }



        file << "#ifndef __ZONE_BB_H__\n#define __ZONE_BB_H__\n\n";
        file << "#define total_BB (" << list_polygons_total_poly << ")\n\n";
        file << "typedef struct _grids\n{\nint PointX;\nint LengthX;\nint PointY;\nint LengthY;\nint PointZ;\nint LengthZ;\nchar Mask;\nshort SlopeRXAngle;\nshort SlopeRZAngle;\n} grids;\n\n";
        file << "const grids const zone_bb[]=\n{\n";

        for (int i = 0; i < list_polygons_total_poly; i++)
        {
            file << "{" << (poly_bb[i].MinValue.x) << ", " << ((poly_bb[i].MaxValue.x-poly_bb[i].MinValue.x)) << ", " << (poly_bb[i].MinValue.y) << ", " << ((poly_bb[i].MaxValue.y-poly_bb[i].MinValue.y)) << ", " << (poly_bb[i].MinValue.z) << ", " << ((poly_bb[i].MaxValue.z-poly_bb[i].MinValue.z)) << ", ";
           // file << "{" << (poly_bb[i].MinValue.x>>16) << ", " << ((poly_bb[i].MaxValue.x-poly_bb[i].MinValue.x)>>16) << ", " << (poly_bb[i].MinValue.y>>16) << ", " << ((poly_bb[i].MaxValue.y-poly_bb[i].MinValue.y)>>16) << ", " << (poly_bb[i].MinValue.z>>16) << ", " << ((poly_bb[i].MaxValue.z-poly_bb[i].MinValue.z)>>16) << ", ";
            file << poly_bb[i].MaskID  <<", " << poly_bb[i].Rotation.x << ", " << poly_bb[i].Rotation.z << "},\n";
        }
        file << "};";
        file << "\n\n//************************************************\n//TOTAL VALUES :  " << list_polygons_total_poly << "\n";


        /*
        for (int refine = 0; refine < list_polygons_total_poly; refine++)
        {
                if (refine_poly_2_bb_list(&poly_bb[refine]) > 0)
                {
                      //  poly_bb[refine].MaxValue.x = poly_bb[refine+1].MaxValue.x; poly_bb[refine].MaxValue.y = poly_bb[refine+1].MaxValue.y; poly_bb[refine].MaxValue.z = poly_bb[refine+1].MaxValue.z;
                      //  poly_bb[refine].MinValue.x = poly_bb[refine+1].MinValue.x; poly_bb[refine].MinValue.y = poly_bb[refine+1].MinValue.y; poly_bb[refine].MinValue.z = poly_bb[refine+1].MinValue.z;
                }
        }*/

    file << "\n#endif";
    file.close();


    }
}



//#define TO_HEX(i) (i <= 9 ? '0' + i : 'A' - 10 + i)




int main()
{
    stringstream name;
    total_mesh = 0;
    /*********************
    The "display" function in JO ENGINE files generated using the Map Editor will be used here to make a list of all the meshes in the file.
    *********************/
    display_fpslevel_mesh();



    cout << "TOTAL MESH : " << total_mesh << "\n"; //Amount of meshes, as returned by the display command.
    int MeshID = 0;

    name << "TESTING_BB_" << MeshID; //For a filename, you just need to send it along a function such as poly_2_bb



    //generate_BSP_hitbox();
    //generate_BB_hitbox(name.str());

    //readBin(name.str());

    //cout << "\nATTEMPTING TO READ FOR VALIDATION :\n";
    //readBin("MeshDATA");

poly_2_BB("NONAMEYET");

write_map_to_bin("OUT/BIN/MeshDATA");

write_MAP_HEADER("OUT/MAPHEADER.h",1, total_mesh);  //Level ID = order of the level (level 1, 2, 3, etc.) Total Mesh is already working fine, but for now we just pass the parameter for debugging)

/***************
WIP, Reads an Obj file, convert it to SGL format and then write it.
***************/
total_mesh=Obj2SGL("IN/FPSLEVEL.obj", 100.0); //File you want to convert (in), and if you want to scale it put the scale factor, else put 1.0 to keep a 1:1 ratio
write_MAP_HEADER("OUT/TESTTEST.h", 1, total_mesh);
write_map_to_bin("OUT/BIN/01/MAPT");

    return 777;
}
